require "mymath"

-- draw text of given color
function draw_text(s,x,y,c)
    local c = c or color.white
    if cfg.textsh then
        lg.setColor(color.shadow)
        lg.print(s, x+1, y+1)
    end
    lg.setColor(c)
    lg.print(s, x, y)
end

-- draw list
function draw_list(l,x,y,c)
    local c = c or color.white
    local i,v
    for i,v in ipairs(l) do
        draw_text(v,x,y+fonth*(i-1),c)
    end
end

-- plays a list of messages sequentially
function play_messages(m)
    for i = 1,#m do
        if not m[i].over then
            if not m[i].started then m[i]:init() end
			draw_window(box.x,box.y,w[i]+15,10*#m)	--< ADDED
            m[i]:play(box.x+10,box.y+10,m[i].wait)
            break
        end
    end
end

function wrap(str)			-- wraps printingc-- non-functional
		local max_line_length = 0.6*string.gmatch(con.output.font,"%d+")()
		local limit = box.w / max_line_length
		local cut = 1
		con.print(""..str:gsub( "(%s+)()(%S+)()",
                          function(sp, st, word, fi)
                            if fi - cut > limit then
                              cut = st
                              return "\n"..word
                            end
                          end ))
		end

-- removes formatting commands from string
function unformat(s)
    s = s:gsub('|n','')
    s = s:gsub('|,','')
    s = s:gsub('|:','')
    s = s:gsub('|!','')
    s = s:gsub('|c%x%x%x%x%x%x%x%x','')
    return s
end

-- bloc class
--[[ this is used by the 'message' class. when a message is created,
it is divided into blocs, each of which has one color and no line
breaks. when the message is played, it is displayed in blocs to
improve performance. --]]

bloc = {}
bloc.__index = bloc

function bloc:new(s,x,y,c)
    --[[
    s: string
    c: color, format |cRRGGBBAA where the values are hexadecimal
    x,y: position
    --]]
    local o = {}
    setmetatable(o,bloc)
    o.c = c or color.white
    local _,r,g,b,a,col
    local n,e = 0,1
    o.s = unformat(s)
    o.nc = o.s:len()
    o.x,o.y = x,y
    -- set color
    if s:sub(1,2) == '|c' then
        _,_,r,g,b,a = s:find('(%x%x+)(%x%x+)(%x%x+)(%x%x+)',3)
        col = {tonumber(r,16),tonumber(g,16),tonumber(b,16),tonumber(a,16)}
        n = n+10
        o.c = col
    end
    return o
end

function bloc:draw(x,y,nc)
    local nc = nc or -1
    draw_text(self.s:sub(1,nc), x+self.x, y+self.y, self.c)
end

-- message class
--[[ this takes a formatted string and parses it so that it can be
displayed in a certain way, i.e. with colored text, pauses, newlines,
and keypresses.

The following formatting options are implemented:
|cRRGGBBAA changes the text color. each two digit number is in hexadecimal
    i.e. "|cff0000ff is text is in red"
|! asks the played to press the "z" key (or whatever key.ent is) before moving on
|n starts a new line
|, pauses the text for the time required to display 10 characters
|: pauses the text for the time required to display 50 characters
    I might change this later on to let you pause for an arbitrary time (in number
    of characters), but what's here feels sufficient.

The arguments are:
s is the formatted string to display.
w is the width the message will wrap at. set this to nil to disable wrapping.
wait is the wait time after the full message displays. set to nil to just press a key and not wait.
skip is whether you want the message to be skipable or not. set to true or false.
--]]
message = {}
message.__index = message

function message:new(s,w,wait,skip)
    local o = {}
    setmetatable(o,message)
    local x,y = 0,0
    local xc,yc = x,y
    local b,z,bw,dx,dy
    local i,j,e = 1,1,1
    local cc = color.white

    --[[ if width 'w' is specified, this adds newlines '|n' at the
    right places so that no line is greater than the specified width.
    --]]
    if w then
        s = s .. ' '
        local ss = ''
        local si,sj,t
        local bi,bj = 1,1
        b = unformat(s)
        j = 1
        t = {['|c']=10, ['|,']=2, ['|:']=2, ['|!']=2}
        for i = 1,s:len() do
            if s:sub(i,i) == ' ' then
                if si and sj and fontw(b:sub(bj,j-1)) > w then -- there must be at least one ' ' before this
                    ss = ss .. s:sub(bi,si-1) .. '|n'
                    bi = si+1
                    bj = sj+1
                end
                si = i
                sj = j
            end
            z = s:sub(i,i+1)
            if not (t[z] == nil) then
                j = j-t[z]
            end
            j = j+1
        end
        ss = ss .. s:sub(bi,s:len())
        s = ss
    end

    --[[ adds pauses in a hackish way, which is by adding spaces
    before the pause command and adjusting the position of the
    displayed block. adding n spaces will be a pause of n/msgspd
    seconds --]]
    local ps = string.rep(' ',10)
    local pl = string.rep(' ',50)
    s = s:gsub('|,',ps..'|,')
    s = s:gsub('|:',pl..'|:')
    o.b = {}
    o.us = unformat(s)
    o.tt = (#o.us)/cfg.msgspd
    o.wait = wait
    o.skip = skip
    i,j = 1,1

    while true do
        b = ''
        z = s:sub(i,i+1)

        -- a new bloc starts when a formatting command is encountered
        if z == '|!' then
            b = s:sub(j,i-1)
            dx,dy = fontw(unformat(b)),0
        elseif z == '|c' then
            b = s:sub(j,i-1)
            dx,dy = fontw(unformat(b)),0
        elseif z == '|n' then
            b = s:sub(j,i-1)
            dx,dy = -xc,fonth
        elseif z == '|,' then
            b = s:sub(j,i-1)
            dx,dy = -fontw(ps)+fontw(unformat(b)),0
        elseif z == '|:' then
            b = s:sub(j,i-1)
            dx,dy = -fontw(pl)+fontw(unformat(b)),0
        end

        -- reached the end
        if i == s:len() then
            b = s:sub(j,i)
            dx,dy = fontw(unformat(b)),0
        end

        -- create bloc if necessary
        if b ~= '' then
            o.b[e] = bloc:new(b,xc,yc,cc)
            cc = o.b[e].c

            -- keypress stuff
            o.b[e].inp = (z == '|!')
            if (z == '|!') then
                o.b[e].t0 = 0
                o.b[e].tset = false
                o.b[e].done = false
            end

            xc,yc = xc+dx,yc+dy
            e,j = e+1,i
        end

        if i == s:len() then break end
        i = i+1

    end
    return o
end

-- init message
function message:init()
    self.ci = 1
    self.done = false
    self.over = false
    self.started = false
    self.t0 = get_time()
end

-- play message at (x,y)
function message:play(x,y,wait)
    local i, tc
    local cnc = 0
    -- first figure out how many chars to display
    local nc = math.round(cfg.msgspd*(get_time()-self.t0),0)
    if not self.started then self.started = true end
    -- message is over, don't display
    if self.done and not self.over then
        if wait == nil then
            if kp == key.ent then self.over = true end
        elseif (get_time()-self.t0 > wait) then
            self.over = true
        end
    end
    if self.over then return end

    -- press enter key to skip the message
    if kp == key.ent and self.skip and (not self.b[self.ci].inp or (self.b[self.ci].inp and (not self.b[self.ci].tset or self.b[self.ci].done))) then
        for i = 1,len(self.b) do
            if self.b[i].inp then
                self.b[i].done = true
            end
        end
        self.done = true
        self.t0 = get_time() -- -(#self.us)/cfg.msgspd
    end

    for i = 1,len(self.b) do
        if nc <= (self.b[i].nc + cnc) and not self.done then -- display the current bloc
            self.b[i]:draw(x,y,nc-cnc)
            self.ci = i
            return
        else
            self.b[i]:draw(x,y) -- display previous (completed) blocs
            cnc = cnc+self.b[i].nc
            if not self.b[i].inp and i == len(self.b) and not self.done then
                self.done = true
                self.t0 = get_time()
                break
            end
            if self.b[i].inp and not self.b[i].done then -- keypress
                self.ci = i
                if self.b[i].tset then
                    if kp == key.ent then -- if player pressed input key
                        self.b[i].done = true
                        if i == len(self.b) and not self.done then
                            self.done = true
                            self.t0 = get_time()
                            break
                        end
                        self.t0 = self.t0 + (get_time()-self.b[i].t0)
                    end
                    break
                else
                    --[[ store time when message paused for keypress.
                    this is later used to adjust the initial time of
                    the message. --]]
                    self.b[i].t0 = get_time()
                    self.b[i].tset = true
                    break
                end
            end
        end
    end
end
